*******************************
*Multiple Imputation 
*October 23, 2013
*StatLab@UVa Library
*Michele Claibourn
*******************************

use "http://people.virginia.edu/~mpc8t/MultipleImputationWorkshop/heartAttack.dta", clear
**1. Examine missingness
describe
misstable summ, gen(miss_) /*summaries about missingness; gen(miss_) creates indictors for missingness*/
misstable patterns /*structure to missingness? monotone pattern? (usually doesn't happen)*/
misstable nested

*Is MCAR reasonable?
/*miss_xxx created above misstable summ, gen(miss_)*/
/*since not significant, assume MCAR, MAR; if model significant, data is not MCAR*/
logit miss_bmi attack age hsgrad female smokes hightar 
logit miss_age attack bmi hsgrad female smokes hightar
logit miss_smokes attack age bmi hsgrad female hightar

*Likely model
* does listwise deletion
logit attack age bmi hsgrad female smokes hightar


**2. Prepare for imputation
*mi set data and register variables
*wide is faster computationally
mi set wide
mi register imputed smokes age bmi hightar /*variables to impute: 4 total*/
mi register regular attack hsgrad female /*leave these alone*/
mi describe

*Check imputation models
* run with no imputations to ensure convergence
hist age
hist bmi
regress age bmi hsgrad female smokes hightar attack
regress bmi age hsgrad female smokes hightar attack
logit hsgrad age bmi female smokes hightar attack
logit female age bmi hsgrad smokes hightar attack
logit smokes age bmi hsgrad female hightar attack
logit hightar age bmi hsgrad female smokes attack


*Dry run -- make sure equations are set up as you intend before implementing
mi impute chained (regress) age bmi (logit) smokes (logit, conditional(if smokes==1) omit(i.smokes)) /// 
	hightar = attack hsgrad female, dryrun

* same as before except after comma	
**3. Impute Data, 10 imputations; set seed for reproducibility; cycle through steps 20 times
mi impute chained (regress) age bmi (logit) smokes (logit, conditional(if smokes==1) omit(i.smokes))  /// 
	hightar = attack hsgrad female, add(10) rseed(97) savetrace(heart1trace, replace) burnin(20)
*   savetrace(...)      save summaries of imputed values from each iteration in filename.dta
* heart1trace.dta saved to working directory
	
**4. Check imputations
*Compare imputed to observed
mi xeq: tab smokes /*see all*/
mi xeq 0: tab smokes /*see m=0*/
mi xeq 1/5: tab smokes if miss_smokes==1 

*More conveniently
foreach var of varlist smokes hightar {
mi xeq 0: tab `var'
mi xeq 1/5: tab `var' if miss_`var'
}

foreach var of varlist age bmi {
mi xeq 0: summ `var'
mi xeq 1/6: summ `var' if miss_`var'
}

mi xeq 0: kdensity bmi; sleep 1000
mi xeq 1/5: kdensity bmi if miss_bmi; sleep 1000

*Or
mi xeq 1/5: twoway (kdensity bmi if miss_bmi==0) || (kdensity bmi if miss_bmi==1), /// 
	legend(label(1 "Observed") label(2 "Imputed"))
	
*Or install midiagplots
*findit midiagplots; user-written function
midiagplots age, m(1) plottype(cumul)
midiagplots bmi, m(1/10) plottype(kdensity)
midiagplots age, sample(observed imputed) plottype(hist)

*Check iterations for convergence
*Need to use previously saved tracefile (heart1trace); looking for long term trends above/below some midpoint (bad)
preserve /*preserves the data, guaranteeing that data will be restored after program termination.*/
use heart1trace, replace 

/*Convert data from wide to long form and vice versa*/
reshape wide *mean *sd, i(iter) j(m) 
* i(iter) = one row for each iter 
* j(m) = stick the value of m on the end of *mean and *sd
tsset iter /* Declare data to be time-series data*/
/*want the next one to bounce around*/
tsline bmi_mean*, title("Mean of Imputed Values of BMI") note("Each line is for one imputation") legend(off)
graph export "C:\Users\jcf2d\Documents\workshops\Multiple Imputation\bmitrace.png", replace

tsline bmi_mean1, title("Mean of Imputed values of BMI, M1")
graph export "C:\Users\jcf2d\Documents\workshops\Multiple Imputation\bmitrace1.png", replace
restore /*forces a restore of the data now.*/


**5. Estimate and combine results
mi estimate: logit attack age bmi hsgrad female smokes hightar 
mi estimate: logit attack age bmi hsgrad female#smokes hightar
mi estimate: logit attack age hsgrad (c.bmi smokes)#female hightar
*Should consider imputing with an interaction between female and smokes/bmi; or imputing separately by female...

