*******************************
*Multiple Imputation 
*Supplemtary
*October 23, 2013
*StatLab@UVa Library
*Michele Claibourn
*******************************

*Generate examples demonstrating bias and loss of power
*Adapted from http://www.ssc.wisc.edu/sscc/pubs/mi/power.do

clear all
set more off
**MCAR, loss of power
*Generate random data
*x1-x5 drawn independently from standard normal distribution
*y is sum of x's, plus normal error term
set obs 1000
set seed 101706
forval i=1/5 {
	gen x`i'=invnorm(runiform())
}
egen y=rowtotal(x*)
replace y=y+8*invnorm(runiform())

summ x*
sjlog using "C:\mpc\DataServices\Workshops\MissingDataWorkshop\missingMCAR", replace
*"True" model
reg y x*
sjlog close, replace

*Drop values at random, about 10% of each variable
forval i=1/5 {
	replace x`i'=. if runiform()<.1
}

summ x*
sjlog using "C:\mpc\DataServices\Workshops\MissingDataWorkshop\missingMCAR", append
*Model using listwise deletion
reg y x*
sjlog close, replace

*Impute the missing values
*But first, note the imputation models themselves are not very predictive
unab vars: *
foreach x of varlist x* {
	local xvars: list vars - x
	reg `x' `xvars'
}

*Impute anyway
mi set wide
mi register imputed x*
mi register regular y
mi impute chained (regress) x*=y, add(10)

*Results after multiple imputation
mi xeq: summarize
mi estimate: reg y x*


**MAR, bias
clear all
set more off
*Generate random data (same as above)
set obs 1000
set seed 101706
forval i=1/5 {
	gen x`i'=invnorm(runiform())
}
egen y=rowtotal(x*)
replace y=y+8*invnorm(runiform())

sjlog using "C:\mpc\DataServices\Workshops\MissingDataWorkshop\missingMAR", replace
*"True" model
reg y x*
sjlog close

*Drop values of x1, dependent on y (observed)
replace x1=. if runiform()<y

summ x*
sjlog using "C:\mpc\DataServices\Workshops\MissingDataWorkshop\missingMAR", append
*Model using listwise deletion
reg y x*
sjlog close

*Impute the missing values
mi set wide
mi register imputed x1
mi register regular x2 x3 x4 x5 y
mi impute chained (regress) x1=y x2 x3 x4 x5, add(10)

*Results after multiple imputation
mi xeq: summarize
mi estimate: reg y x*


**Generate figures for reduction in variance, bias in covariance
**(Conditional) Mean imputation
clear all
set more off

*Generate random data
*x1-x10 drawn independently from standard normal distribution
*y is sum of x's, plus normal error term
set obs 1000
set seed 17
forval i=1/3 {
	gen x`i'=invnorm(runiform())
}
egen y=rowtotal(x*)
replace y=y+5*invnorm(runiform())

*"True" model
reg y x*

*Drop 10% of y variable
replace y=. if runiform()<.1

*Mean imputation
egen yavg=mean(y)
gen ymean=y
replace ymean=yavg if y==.
scatter ymean x1 if y!=., mcolor(black) || scatter ymean x1 if y==., mcolor(blue)
graph export "C:\mpc\DataServices\Workshops\MissingDataWorkshop\scatterMean.png", replace

*Regression imputation
regress y x1
predict yhat, xb
gen yreg=y
replace yreg=yhat if y==.
scatter yreg x1 if y!=., mcolor(black) || scatter yreg x1 if y==., mcolor(blue)
graph export "C:\mpc\DataServices\Workshops\MissingDataWorkshop\scatterReg.png", replace
